
import os
import base64
import logging
import inspect

import pyaas


def generateCookieSecret(path):
    secret = base64.b64encode(os.urandom(32))
    with open(path, 'w') as fp:
        fp.write(secret)
    return secret


def getParent():
    # inspect who called this function
    frames = inspect.getouterframes(inspect.currentframe())
    # get the caller frame
    frame = frames[-1]
    # get the path of the caller
    script = os.path.abspath(frame[1])

    return script





class CacheTx(object):

    def __enter__(self):
        return self

    def __exit__(self):
        self.end()

    def put(self, key, field, value):
        raise NotImplementedError()

    def remove(self, key, field):
        raise NotImplementedError()

    def end(self):
        raise NotImplementedError()


class Cache(object):

    def start(self):
        raise NotImplementedError()

    def put(self, key, field, value):
        raise NotImplementedError()

    def get(self, key, field):
        raise NotImplementedError()

    def remove(self, key, field):
        raise NotImplementedError()

    def removeall(self, key):
        raise NotImplementedError()

    def getall(self, key):
        raise NotImplementedError()



import logging

import pyaas

import tornado.web

try:
    import ldap
except ImportError:
    raise pyaas.error('Missing LDAP module')


class Slap(tornado.web.RequestHandler):
    def get(self):
        next = self.get_argument('next', '/')
        self.render('login.html', next=next)

    def post(self):
        username = self.get_argument('username', '')
        username = tornado.escape.xhtml_escape(username)
        password = self.get_argument('password', '')

        ldap_dn  = pyaas.config.get('slap', 'dn')
        ldap_uri = pyaas.config.get('slap', 'uri')

        try:
            dn = ldap_dn.format(username)
            ldap_server = ldap.initialize(ldap_uri)
            ldap_server.bind_s(dn, password)
            ldap_server.unbind()

            self.set_secure_cookie('uid', username)

        except ldap.INVALID_CREDENTIALS:
            logging.warn('Invalid credentials for user: %s', username)

        except ldap.SERVER_DOWN:
            logging.warn('Could not connect to LDAP server: %s', ldap_uri)

        self.redirect(self.get_argument('next', '/'))




class Database:
    def __init__(self, **kwds):
        pass

    def Initialize(self):
        return

    def Sync(self):
        return

    def Find(self, table, params=None, sort=None):
        return []

    def FindOne(self, table, _id):
        return {}

    def Count(self, table):
        return 0

    def Insert(self, table, values):
        return

    def Update(self, table, values):
        return

    def Remove(self, table, _id):
        return




import collections
import logging

import pyaas

moduleImports = dict(
    auth    = 'pyaas.web.auth',
    storage = 'pyaas.storage.engines',
    cache   = 'pyaas.storage.cache',
    )


def load():
    modules = dict(pyaas.config.items('modules'))
    for module in modules:
        try:
            modulePath = pyaas.module.moduleImports[module]
        except KeyError:
            logging.error('Unknown module: %s', module)
            continue

        # on import PyaasModules register themselves
        __import__(pyaas.module.moduleImports[module])

    for module, moduleClass in pyaas.module.PyaasModule.registry.items():
        moduleClass.load()

    return


class RegisterModuleMeta(type):
    def __init__(cls, name, bases, dct):
        if not hasattr(cls, 'registry'):
            # this is the base class.  Create an empty registry
            cls.registry = {}
        else:
            # this is a derived class.  Add cls to the registry
            cls.registry[name] = cls

        super(RegisterModuleMeta, cls).__init__(name, bases, dct)


class PyaasModule(object):
    PKG_PATH = 'pyaas'
    CLASSES = collections.defaultdict(dict)

    __metaclass__ = RegisterModuleMeta

    @classmethod
    def load(cls):
        raise NotImplementedError

    @classmethod
    def loadModule(cls, moduleName):
        classes = cls.CLASSES[cls.__name__]

        try:
            return classes[moduleName]
        except KeyError:
            # try to load the module
            pass

        # then try loading a pyaas module first
        try:
            path = cls.PKG_PATH + '.' + moduleName
            module = __import__(path)
        except ImportError:
            # try loading a user-supplied module next
            try:
                path = moduleName
                module = __import__(path)
            except ImportError:
                raise pyaas.error('Unknown module: %s', moduleName)

        subPackageName = path
        for subPackageName in subPackageName.split('.')[1:]:
            module = getattr(module, subPackageName)

        classname = subPackageName.capitalize()

        moduleClass = getattr(module, classname, None)
        if moduleClass is None:
            try:
                moduleClass = getattr(module, 'Database')
            except AttributeError:
                raise pyaas.error('Bad module: %s', moduleName)

        classes[moduleName] = moduleClass

        return moduleClass




import logging

import pyaas

class StorageModule(pyaas.module.PyaasModule):
    PKG_PATH = 'pyaas.storage.engines'

    @classmethod
    def load(cls):
        if not pyaas.config.has_option('modules', 'storage'):
            return

        instances = pyaas.config.get('modules', 'storage')
        instances = [instance.strip() for instance in instances.split(',')]
        for instance in instances:
            logging.info('Loading storage instance: %s', instance)

            if not pyaas.config.has_section(instance):
                logging.warn('No configuration for instance: %s', instance)
                continue

            if not pyaas.config.has_option(instance, 'engine'):
                logging.warn('No engine specified for instance: %s', instance)
                continue

            params = dict(pyaas.config.items(instance))

            engine = params.pop('engine')

            engineClass = cls.loadModule(engine)

            # convert user supplied engine name into a single value
            section = engine.split('.')[-1]

            if pyaas.config.has_section(section):
                options = dict(pyaas.config.items(section))
            else:
                options = {}

            options.update(params)

            isCritical = options.pop('critical', 'true')
            try:
                pyaas.storage.databases[instance] = engineClass(**options)
            except pyaas.error as e:
                if isCritical.lower() in ['false', 'f' 'no', 'n', '0']:
                    logging.warn('Unable to instantiate engine: %s:%s', engine, instance)
                    logging.debug('%s', e)
                else:
                    raise

        return




import sys
import os
import argparse
import collections
import logging

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import pyaas

pyaas.argparser = argparse.ArgumentParser()

pyaas.argparser.add_argument('--ini', '-i',
    help='Specify additional ini file')

pyaas.argparser.add_argument('--debug', '-d',
    action='store_true',
    help='Print verbose debugging information')

logging.basicConfig(
    format  = '%(asctime)s %(levelname)-8s %(message)s',
    datefmt = '%Y-%m-%d %H:%M:%S',
    level   = logging.INFO
)



def load(settings=None, namespace=None, prefix=None):
    """
    Call this guy to init pyaas stuffs
    :param settings: Alternative name of ini file to load
    :param namespace: Namespace is used to derive paths, pass '' for an empty namespace
    :param prefix: The root path of the app
    :return: None
    """

    parent = pyaas.util.getParent()

    if prefix is None:
        # get the filename of the caller
        # get the directory name of the file
        prefix = os.path.dirname(parent)

        if prefix.endswith(os.path.sep + 'bin'):
            prefix = os.path.join(prefix, '..')
            prefix = os.path.abspath(prefix)

    prefix = os.path.abspath(prefix)
    if pyaas.prefix != prefix:
        pyaas.prefix = prefix
        logging.debug('Setting prefix to "%s"', pyaas.prefix)

    if namespace is None:
        namespace = os.path.basename(parent)
        namespace = namespace.rsplit('.', 1)[0]

    if namespace != pyaas.namespace:
        pyaas.namespace = namespace
        logging.debug('Setting namespace to "%s"', pyaas.namespace)

    # if settings is not passed in use the supplied or derived namespace
    settings = settings or namespace

    pyaas.args = pyaas.argparser.parse_args()

    pyaas.config = configparser.SafeConfigParser(dict_type=collections.OrderedDict)
    pyaas.config.optionxform = str

    ini_files = [
        pyaas.paths('etc', settings + '.ini'),
        pyaas.paths('etc', settings + '.ini.local')
    ]

    if pyaas.args.ini:
        ini_files.append(pyaas.args.ini)

    try:
        ok = pyaas.config.read(ini_files)
    except configparser.ParsingError as e:
        raise pyaas.error('Unable to parse file: %s', e)

    if not ok:
        raise pyaas.error('Unable to read config file(s): %s', ini_files)

    # setup file log
    logfile = logging.FileHandler(pyaas.paths('var', settings + '.log'))
    logfile.setLevel(logging.INFO)

    logfile.setFormatter(
        logging.Formatter(
            fmt = '%(asctime)s %(levelname)-8s %(message)s',
            datefmt = '%Y-%m-%d %H:%M:%S',
            )
        )

    # add the handlers to the logger
    root = logging.getLogger()
    root.addHandler(logfile)

    if pyaas.args.debug:
        root.setLevel(logging.DEBUG)
        logfile.setLevel(logging.DEBUG)

    # call this here if there is no daemon option
    if not hasattr(pyaas.args, 'daemon'):
        pyaas.module.load()

    return




import pyaas

try:
    import redis
except ImportError:
    raise pyaas.error('Missing redis module')


class CacheTx(object):

    def __init__(self, redis):
        self._pipeline = redis.pipeline()

    def __enter__(self):
        return self

    def __exit__(self):
        self.end()

    def put(self, key, field, value):
        self._pipeline.hset(key, field, value)
        return self

    def remove(self, key, field):
        self._pipeline.hdel(key, field)
        return self

    def end(self):
        self._pipeline.execute()


class Pyredis(object):

    def __init__(self, **kwargs):
        self._redis = redis.StrictRedis(**kwargs)

    def start(self):
        return CacheTx(self._redis)

    def put(self, key, field, value):
        self._redis.hset(key, field, value)

    def get(self, key, field):
        return self._redis.hget(key, field)

    def remove(self, key, field):
        self._redis.hdel(key, field)

    def removeall(self, key):
        self._redis.delete(key)

    def getall(self, key):
        return self._redis.hgetall(key)




import tornado.web

from pyaas.web.handlers import Base


class Index(Base):
    def initialize(self, template='index'):
        self.template = template + '.html'

    def get(self, template=None):
        template = template + '.html' if template else self.template
        self.render(template)


class Protected(Base):
    def initialize(self, template='index'):
        self.template = template + '.html'

    @tornado.web.authenticated
    def get(self, template=None):
        template = template + '.html' if template else self.template
        self.render(template)




from .application import Application

import pyaas

pyaas.argparser.add_argument('--name',
    help='Specify a name for the instance')

pyaas.argparser.add_argument('--address', '-a',
    help = 'Interface to bind to')

pyaas.argparser.add_argument('--port', '-p',
    type=int,
    help='Port to bind to')

pyaas.argparser.add_argument('--newcookie',
    action='store_true',
    help='Generate a new cookie')




import os
import socket
import logging

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import pyaas
import pyaas.web.handlers

import tornado.web
import tornado.ioloop

pyaas.ioloop = tornado.ioloop.IOLoop.instance()


class Application(tornado.web.Application):
    def __init__(self, section='server'):
        self.websockets = set()

        section = pyaas.args.name or section

        if not pyaas.config.has_section(section):
            raise pyaas.error('Invalid instance name: %s', section)

        # get the interface and port to listen on
        self.addr = pyaas.args.address or pyaas.config.get(section, 'address')
        self.port = pyaas.args.port or pyaas.config.getint(section, 'port')

        # load the cookie secret used to encrypt cookies
        cookie_path = pyaas.paths('etc', 'cookie.secret')

        if pyaas.args.newcookie:
            cookie_secret = pyaas.util.generateCookieSecret(cookie_path)
        else:
            try:
                with open(cookie_path, 'rb') as fp:
                    cookie_secret = fp.read(44)
            except IOError:
                cookie_secret = pyaas.util.generateCookieSecret(cookie_path)

        # URI patterns
        if not hasattr(self, 'ssl_options'):
            self.ssl_options = None

        # URI patterns
        if not hasattr(self, 'patterns'):
            self.patterns = []

        # Tornado settings
        self.settings = dict(
            static_path   = pyaas.paths('share', 'static'),
            template_path = pyaas.paths('share', 'templates'),
            cookie_secret = cookie_secret,
            debug         = False
        )

        # useful during development
        if pyaas.args.debug:
            self.settings['debug'] = True

        authModules = pyaas.module.PyaasModule.CLASSES.get('AuthModule', None)
        if authModules:
            for (name,authModule) in authModules.items():
                # extend the patterns and settings accordingly
                self.patterns.extend([
                    ( authModule.URL, authModule ),
                    ( r'/logout', pyaas.web.auth.Logout ),
                    ])

            logging.debug('Setting default login URL to: %s', authModule.URL)
            self.settings['login_url'] = authModule.URL


    def Listen(self):
        # initialize here so patterns and settings can be extended by plugins
        tornado.web.Application.__init__(self, self.patterns, **self.settings)

        try:
            self.listen(self.port, self.addr, xheaders=True, ssl_options=self.ssl_options)
        except socket.gaierror as e:
            if 8 == e.errno:
                raise pyaas.error('Invalid address specified "%s"' % self.addr)
            raise pyaas.error('Could not listen: %s' % e)
        except socket.error as e:
            raise pyaas.error('Could not listen: %s' % e)

        logging.info('Listening on %s:%d', self.addr, self.port)

        pyaas.ioloop.start()

    def Broadcast(self, msg):
        'Broadcast a message to all connected sockets'

        for client in self.websockets:
            client.write_message(msg)

    def Stop(self):
        pyaas.ioloop.stop()






import os

from .version import VERSION

from . import util
from . import module
from . import settings
from . import module

# a generic error class for throwing exceptions
class error(Exception):
    def __init__(self, fmt, *args):
        self.str = fmt % args

    def __str__(self):
        return self.str


class Paths(object):
    def __getattr__(self, attribute):
        try:
            return object.__getattribute__(self, attribute)
        except AttributeError:
            return os.path.join(prefix, attribute, namespace)

    def __call__(self, toplevel, *args):
        return os.path.join(prefix, toplevel, namespace, *args)


paths       = Paths()
prefix      = ''
namespace   = ''
cache       = None



import pyaas


def cache_record_result(func):
    def _record_result(*args, **kwargs):

        # check cache
        if pyaas.cache:
            k = 'Cache:Record:' + args[1]
            r = pyaas.cache.get(k, args[2])
            if r is not None:
                return r

        r = func(*args, **kwargs)

        # cache it
        if pyaas.cache:
            pyaas.cache.put(k, args[2], r)

        return r
    return _record_result

def cache_record_remove(func):
    def _record_remove(*args, **kwargs):
        r = func(*args, **kwargs)

        # drop it
        if pyaas.cache:
            k = 'Cache:Record:' + args[1]
            pyaas.cache.remove(k, args[2][args[3]])

        return r

    return _record_remove

def cache_result(func):
    def _result(*args, **kwargs):

        # check cache
        if pyaas.cache:
            k = 'Cache:Result:' + args[1]
            r = pyaas.cache.get(k, 'result')
            if r is not None:
                return r

        r = func(*args, **kwargs)

        # cache it
        if pyaas.cache:
            pyaas.cache.put(k, 'result', r)

        return r

    return _result


def cache_remove(func, key):
    def _remove(*args, **kwargs):

        r = func(*args, **kwargs)

        if pyaas.cache:
            pyaas.cache.remove(key, '__field')

        return r
    return _remove



# login methods are dynamically imported if auth is enabled

import logging

from .logout import Logout

import pyaas

class AuthModule(pyaas.module.PyaasModule):
    PKG_PATH = 'pyaas.web.auth'

    @classmethod
    def load(cls):
        if not pyaas.config.has_option('modules', 'auth'):
            return

        instances = pyaas.config.get('modules', 'auth')
        instances = [instance.strip() for instance in instances.split(',')]
        for instance in instances:
            logging.info('Loading authentication: %s', instance)

            if not pyaas.config.has_section(instance):
                logging.warn('No configuration for authentication: %s', instance)
                continue

            params = dict(pyaas.config.items(instance))

            url = params.pop('login', '/login')

            authClass = cls.loadModule(instance)
            authClass.URL = url

        return


#class Auth(PyaasModule):
#    """
#    Pyaas auth module
#    """
#    def __init__(self):
#        super(Auth, self).__init__('auth', 'pyaas.web.auth', 'Login')
#
#    def load(self, application=None):
#        login = self.getClass()
#        if login:
#            # extend the patterns and settings accordingly
#            application.patterns.extend([
#                ( r'/login',  login                 ),
#                ( r'/logout', pyaas.web.auth.Logout ),
#                ])
#
#            application.settings['login_url'] = '/login'
#
#            logging.debug('Enabled authentication: %s', self.implName)



import sys
import logging

import pyaas

try:
    import motor
    #import pymongo
    #import gridfs
    import bson
    import bson.json_util
except ImportError:
    raise pyaas.error('Missing motor (mongodb) module')


class Mongo:
    def __init__(self, server, database, store=None):
        self.mongo = motor.MotorClient(server)

        self.mongo.document_class = bson.SON

        self.db = self.mongo[database]
        if store:
            self.fs = gridfs.GridFS(self.mongo[store])
        else:
            self.fs = None

    def Initialize(self):
        return

    def Sync(self):
        return

    #def Reset(self):
    #    pyaas.mongo.drop_database(self.dbname)
    #    pyaas.mongo.drop_database(self.store)

    def Find(self, table, params=None, sort=None):
        return list(self.db[table].find(params, sort=sort))

    def FindOne(self, table, _id):
        return self.mongo[table].find_one(_id)

    def Count(self, table):
        return 0

    def Insert(self, table, values):
        return

    def Update(self, table, values=None):
        if values:
            self.record.update(values)
        self.db[table].save(self.record)
        return self

    def Remove(self, table, _id):
        self.db[table].remove(self.id)
        return True



__author__ = 'mriley'

import logging
import pyaas

from .decorator import *


class CacheModule(pyaas.module.PyaasModule):
    PKG_PATH = 'pyaas.storage.cache'

    @classmethod
    def load(cls):
        if not pyaas.config.has_option('modules', 'cache'):
            return

        instances = pyaas.config.get('modules', 'cache')
        instances = [instance.strip() for instance in instances.split(',')]
        instances.reverse()
        for instance in instances:
            logging.info('Loading cache: %s', instance)

            if not pyaas.config.has_section(instance):
                params = {}
            else:
                params = dict(pyaas.config.items(instance))

            cache_class = cls.loadModule(instance)
            if pyaas.cache is None:
                pyaas.cache = cache_class(**params)
            else:
                params['next'] = pyaas.cache
                pyaas.cache = cache_class(**params)
        return




class CacheTx(object):

    def __init__(self, mem, next=None):
        self._mem = mem
        self._next = next

    def __enter__(self):
        return self

    def __exit__(self):
        self.end()

    def put(self, key, field, value):
        self._mem.put(key, field, value)
        if self._next is not None:
            self._next.put(key, field, value)
        return self

    def remove(self, key, field):
        self._mem.remove(key, field)
        if self._next is not None:
            self._next.remove(key, field)
        return self

    def end(self):
        if self._next is not None:
            self._next.end()


class Memory(object):

    def __init__(self, next=None):
        self._l1 = {}
        self._next = next

    def start(self):
        if self._next is None:
            return CacheTx(self, None)
        return CacheTx(self, self._next.start())

    def put(self, key, field, value):
        if key not in self._l1:
            self._l1 = {}
        self._l1[key][field] = value
        if self._next is not None:
            self._next.put(key, field, value)

    def get(self, key, field):
        try:
            return self._l1[key][field]
        except KeyError:
            pass
        if self._next is not None:
            if key not in self._l1:
                self._l1 = {}
            obj = self._l1[key][field] = self._next.get(key, field)
            return obj
        return None

    def remove(self, key, field):
        try:
            del self._l1[key][field]
        except KeyError:
            pass
        if self._next is not None:
            self._next.remove(key, field)

    def removeall(self, key):
        try:
            del self._l1[key]
        except KeyError:
            pass
        if self._next is not None:
            self._next.removeall(key)

    def getall(self, key):
        if self._next is not None:
            return self._next.getall(key)
        else:
            try:
                return self._l1[key]
            except KeyError:
                pass
        return {}


#!/usr/bin/env python

import sys
import os
import argparse
import zipfile
import shutil
import time


def replace_in_file(src, dst, **kwargs):
    with open(src) as r:
        with open(dst, 'w') as w:
            for line in r:
                w.write(replace_in_string(line, **kwargs))


def replace_in_string(string, **kwargs):
    for key, value in kwargs.iteritems():
        string = string.replace(key, value)
    return string


def replace_all(filename, **kwargs):
    dst = replace_in_string(filename, **kwargs)
    if os.path.isdir(filename):
        if filename != dst:
            shutil.move(filename, dst)
            filename = dst
        for f in os.listdir(filename):
            replace_all(os.path.join(filename, f), **kwargs)
    else:
        if filename == dst:
            tmp = '%s.tmp.%d' % (filename, time.time())
            shutil.move(filename, tmp)
            filename = tmp
        replace_in_file(filename, dst, **kwargs)
        os.remove(filename)


def main():
    argparser = argparse.ArgumentParser()

    argparser.add_argument('--name', '-n',
        default='example', required=True,
        help='Name of the project to create')

    args = argparser.parse_args()

    parts = args.name.split(' ')
    camel_name = ''.join(p.capitalize() for p in parts)
    lower_name = ''.join(p.lower()      for p in parts)

    dstdir = os.path.basename(lower_name)
    dstdir = os.path.join(os.getcwd(), dstdir)
    dstdir = os.path.abspath(dstdir)

    try:
        os.mkdir(dstdir)
    except OSError as e:
        if 17 == e.errno:
            sys.stderr.write('Directory with that name already exists.\n')
            sys.exit(-1)
        raise

    skel = os.path.dirname(__file__)
    skel = os.path.join(skel, 'skel.zip')
    skel = zipfile.ZipFile(skel, 'r')

    skel.extractall(dstdir)

    os.chdir(dstdir)

    replace_all(dstdir, example=lower_name, Example=camel_name)


if '__main__' == __name__:
    main()




import os

import pyaas

try:
    import psycopg2
    import psycopg2.extras
except ImportError:
    raise pyaas.error('Missing psycopg2 (Postgresql) module')


class Pgsql:
    def __init__(self, **kwds):
        self.schema = kwds.get('schema', None)
        if self.schema is not None:
            del kwds['schema']

        try:
            self.conn = psycopg2.connect(**kwds)
        except psycopg2.OperationalError as e:
            raise pyaas.error('Could not connect to database: %s', e)

        self.cursor = self.conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

    def Initialize(self):
        if self.schema:
            schema = os.path.join(pyaas.prefix, self.schema)
            if schema and os.path.isfile(schema):
                with open(schema, 'rb') as f:
                    schema = f.read()
                    with self.conn as conn:
                        self.cursor.execute(schema)
                        conn.commit()

    def Sync(self):
        pass

    def Execute(self, statement, *args):
        with self.conn as conn:
            self.cursor.execute(statement, args)
            conn.commit()
        return self.cursor.fetchall()

    def Execute2(self, statement, *args):
        with self.conn as conn:
            self.cursor.execute(statement, args)
            conn.commit()

    def Find(self, table, params=None, sort=None):
        statement = 'SELECT * FROM ' + table
        if params:
            statement += ' WHERE ' + params
        if sort:
            statement += ''

        with self.conn as conn:
            self.cursor.execute(statement)
            conn.commit()

        return self.cursor.fetchall()

    def FindOne(self, table, _id, id_column='id'):
        statement = 'SELECT * FROM {0} WHERE {1} = %s'.format(table, id_column)
        with self.conn as conn:
            self.cursor.execute(statement, [_id])
            conn.commit()
        return self.cursor.fetchone()

    def Count(self, table):
        statement = 'SELECT count(*) FROM {0}'.format(table)
        with self.conn as conn:
            self.cursor.execute(statement)
            conn.commit()
        return self.cursor.fetchone()[0]

    def Insert(self, table, values, id_column='id'):
        columns = ','.join('"%s"' % k.lower() for k in values.keys())
        placeholder = ','.join('%s' for x in xrange(len(values)))
        statement = 'INSERT INTO {0} ({1}) VALUES ({2})'.format(table, columns, placeholder)

        with self.conn as conn:
            self.cursor.execute(statement, values.values())
            conn.commit()

        rows = self.cursor.rowcount
        if rows is None:
            rows = -1

        if id_column not in values:
            self.cursor.execute('SELECT lastval()')
            values[id_column] = self.cursor.fetchone()[0]

        return rows

    def Upsert(self, table, values, id_column='id'):
        rows = self.InsertUnique(table, values, id_column)
        if rows <= 0:
            rows = self.Update(table, values, id_column)
        return rows

    def InsertUnique(self, table, values, id_column='id'):
        columns = ','.join('"%s"' % k.lower() for k in values.keys())
        placeholder = ','.join('%s' for x in xrange(len(values)))
        statement = "INSERT INTO {0} ({1}) SELECT {2} WHERE NOT EXISTS (select {4} from {0} where {4} = '{3}')" \
            .format(table, columns, placeholder, values[id_column], id_column)

        with self.conn as conn:
            self.cursor.execute(statement, values.values())
            conn.commit()

        rows = self.cursor.rowcount
        if rows is None:
            rows = -1
        return rows

    def Update(self, table, values, id_column='id'):
        _id = values[id_column]
        columns = ','.join('"%s"=%%s' % s.lower() for s in values.keys())
        statement = 'UPDATE {0} SET {1} WHERE {2}=%s'.format(table, columns, id_column)

        with self.conn as conn:
            self.cursor.execute(statement, values.values() + [_id])
            conn.commit()

        rows = self.cursor.rowcount
        if rows is None:
            rows = -1
        return rows

    def Remove(self, table, _id, id_column='id'):
        statement = 'DELETE FROM {0} WHERE {1} = %s'.format(table, id_column)
        with self.conn as conn:
            self.cursor.execute(statement, [_id])
            conn.commit()



#!/usr/bin/env python

import os
import zipfile

from distutils.core import setup

zf = zipfile.ZipFile(os.path.join('pyaas', 'skel.zip'), 'w')
for base,directories,filenames in os.walk('skel'):
    for filename in filenames:
        path = os.path.join(base, filename)
        zf.write(path, path[5:])
zf.close()

execfile('pyaas/version.py')

setup(
    name='pyaas',
    version=VERSION,
    author='Matthew Oertle',
    author_email='moertle@gmail.com',
    url='https://github.com/moertle/pyaas',
    license='MIT',
    description='Python-as-a-Service is a set of utilities for quickly creating Tornado applications.',
    long_description=open('README.rst').read(),
    packages=[
        'pyaas',
        # web service components
        'pyaas.web',
        'pyaas.web.auth',
        'pyaas.web.handlers',
        # storage components
        'pyaas.storage',
        'pyaas.storage.engines',
        # cache components
        'pyaas.storage.cache',
    ],
    package_data={
        'pyaas': ['skel.zip']
    },
    install_requires=[
        "tornado >= 4.0.1",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Internet :: WWW/HTTP",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
    ],
    zip_safe=False
)



#!/usr/bin/env python

# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# -----------------------------------------------------------------------------

# The above license applies only to the example project. The intent is to allow
# you to freely use this template for your project and license it as you see
# fit. The pyaas library is released under the MIT License.

# -----------------------------------------------------------------------------

# This file can live in the base directory of the project or in a ./bin
# directory and init() (see below) will resolve accordingly

# This bit just ensures that this file isn't on the module search path
import sys
import os

# used for development
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

# used if not using a bin directory
#import inspect
#localpath = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
#sys.path = [p for p in sys.path if p != localpath]

import logging

import pyaas
import pyaas.daemon
import pyaas.storage

import example
import example.app

def runExampleApp():
    # Calling pyaas.settings.load() without args is equivalent to:
    # pyaas.settings.load(prefix='/full/path/to/example', namespace='example', settings='example')
    #
    # Passing an empty string as the namespace trims the paths
    # e.g. etc/example.ini vs etc/example/example.ini
    # -or- share/static/favicon.ico vs share/example/static/favicon.ico

    pyaas.settings.load(namespace='example')
    pyaas.daemon.Daemonize(entry)

def entry():
    pyaas.settings.postinit()

    app = example.app.ExampleApp()
    try:
        app.Listen()
    except KeyboardInterrupt:
        app.Stop()

if '__main__' == __name__:
    try:
        runExampleApp()
    except pyaas.error as e:
        logging.exception('%s', e)




import tornado.web


class Base(tornado.web.RequestHandler):
    def get_current_user(self):
        uid = self.get_secure_cookie('uid')
        if uid is None:
            self.clear_cookie('uid')
        return uid


from .index import *
from .      import ws
